#!/usr/bin/python3.10
# Jinwen Liu <j.liu@lic.leidenuniv.nl>, version 20/12/2023

# INPUT FILE NEEDED : "trajectories.xyz"
# Should contain trajectories of the whole simulation (concatenated file)

### MODULES ###
from os.path import isfile, isdir, join
from os import listdir, chdir
from re import match
import MDAnalysis as mda
import sys
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as mcolors
import matplotlib_parameters
from ase.io.cube import read_cube_data

### PATHWAYS & DEFINITIONS ###
au_to_ev = 2.72113838565563E+01
time_step_collect_energy = 10 # 10*5=50 ps

pre_equilibration_time = 5000
pre_equilibration_fermi = int(pre_equilibration_time / (time_step_collect_energy*5))

### COLLECT TOTAL ENERGIES ###
print('\n### COLLECT TOTAL ENERGIES ###\n')
# Collect data in the concatenated "trajectories.xyz" file
total_energies = []
steps = 0
filename = "trajectories_0.0ML.xyz"
if(isfile(filename)):
    with open(filename) as f:
        content = f.readlines()
        for line in content:
            matching = match(".*E =.*", line)
            if not(matching is None):
                steps += 1
                if steps % time_step_collect_energy == 0:
                    # Fermi energies data
                    total_energies.append(float(line.split()[-1]))
# print(total_energies)
total_energies_avg = sum(total_energies[pre_equilibration_fermi:]) / len(total_energies[pre_equilibration_fermi:])
# Define time steps data
time_steps = np.arange(time_step_collect_energy,len(total_energies)*time_step_collect_energy+time_step_collect_energy,time_step_collect_energy)

# PRINT INFORMATIONS ON TERMINAL
print('Number of time steps during the simulation : ' + str(steps) + '\nNumber of Fermi energies collected : ' + str(len(total_energies)) + '\n')
print('Average total energy after pre-equilibration : ' + str(total_energies_avg))
### PRINT FERMI ENERGIES IN fermi_energies.dat ###
file_out = open("total_energies.dat", "w")
i = 0
while i < len(total_energies):
    file_out.write("%12.10f" % (time_steps[i]) + "    " + "%12.10f" % (total_energies[i]) + "\n")
    i += 1
file_out.close()
print('... Time steps/total energies data printed in file : total_energies.dat\n')

### PLOT FERMI ENERGIES ###
fig, ax = plt.subplots()
color = matplotlib_parameters.diverging2[0]
time_steps = time_steps/1000*5 # fs to ps
ax.plot(time_steps, total_energies, color=color, linewidth=1, linestyle="-", zorder=0, alpha=0.5)
ax.scatter(time_steps, total_energies, marker="o", s=80, edgecolors="black", color=color, zorder=1, alpha=1)
ax.hlines(total_energies_avg, min(time_steps)+5, max(time_steps), color=color, linestyle="--", zorder=2, alpha = 0.5, linewidth=1.5)
# Plot parameters
plot_title = 'total energies collected over the whole simulation'
name_for_saving_plot = 'total_energies.png'
path_for_saving_plot = './'
ax.set(xlabel="Time [ps]", ylabel="total energy [a.u.]")
plt.title(plot_title)
plt.show()

