import numpy as np
from matplotlib import pyplot as plt
from os.path import isfile, join
#
# Functions
def module(x1, x2, y1, y2, z1, z2, x_size, y_size, z_size):
    dx = np.abs(x1 - x2)
    dy = np.abs(y1 - y2)
    dz = np.abs(z1 - z2)
    #
    # PBC
    if (dx > x_size * 0.5):
        dx = dx - x_size

    if (dy > y_size * 0.5):
        dy = dy - y_size

    if (dz > z_size * 0.5):
        dz = dz - z_size
    #
    # Norm
    d = np.sqrt( (dx)**2 + (dy)**2 + (dz)**2 )
    return d

# %% Open the folder

path = './'


# %% Main code
'We must define two parameters, the total number of layers and'
'the number of layers to be considered in the calculation.'
n_layer_tot = 4
n_layer     = 1  # 1 for the surface layer only
nads        = 16 # 16 H max

'We also have to determine the reference coordinates of each atom. To'
'do so, we use their coordinates in the CONTCAR of the plain surface.'
filename = path + "CONTCAR_0"
with open(filename) as f:
    contcar_ref = f.readlines()
    
    x_line = contcar_ref[2].split()
    x_size = float(x_line[0]) 
    y_line = contcar_ref[3].split()
    y_size = float(y_line[1])
    z_line = contcar_ref[4].split()
    z_size = float(z_line[2]) 
    
    nM = int(contcar_ref[6])
    x_ref = np.zeros([nM])
    y_ref = np.zeros([nM])
    z_ref = np.zeros([nM])
    
    coord = contcar_ref[8]
    
    j = 0
    for i in range(9,nM+9):
        L = contcar_ref[i].split()
        x_ref[j] = float(L[0])    
        y_ref[j] = float(L[1])
        z_ref[j] = float(L[2])
        j = j+1
        
    x_ref = x_ref * x_size
    y_ref = y_ref * y_size
    z_ref = z_ref * z_size

avg_M = np.zeros([5,2])
avg_H = np.zeros([5,2])
ipt = 0

'I did not find a way to make this automatic, yet.'
'In the meantime, we must enter manually the list'
'of all the input configurations.'
for k in (1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16):#,6,7,8,10,12,16):
    
    avg_M[ipt,0] = k
    avg_H[ipt,0] = k
    
    # Open file and convert to cartesian coordinates
    'We read the CONTCAR file and we check whether we'
    'need toconvert back into cartesian coordinates.'
    filename = path + "CONTCAR_"  + str(k)
    with open(filename) as f:
        contcar = f.readlines()
    
        x_line = contcar[2].split()
        x_size = float(x_line[0]) 
        y_line = contcar[3].split()
        y_size = float(y_line[1])
        z_line = contcar[4].split()
        z_size = float(z_line[2]) 
        
        number = contcar[6].split()
        nM = int(number[0])
        nH = int(number[1])
        x = np.zeros([nM+nH])
        y = np.zeros([nM+nH])
        z = np.zeros([nM+nH])
        
        coord = contcar[8]
        
        j = 0
        for i in range(9,nM+nH+9):
            L = contcar[i].split()
            x[j] = float(L[0])    
            y[j] = float(L[1])
            z[j] = float(L[2])
            j = j+1
            
        x = x * x_size
        y = y * y_size
        z = z * z_size

    # 'Do not forget the POSCAR for the hydrogen displacement.'
    # filename = path + "POSCAR_" + str(k)
    # with open(filename) as f:
    #     poscar = f.readlines()
        
    #     x_p = np.zeros([nM+nH])
    #     y_p = np.zeros([nM+nH])
    #     z_p = np.zeros([nM+nH])
        
    #     j = 0
    #     for i in range(9+nM,9+nM+nH):
    #         L = poscar[i].split()
    #         x_p[j] = float(L[0])    
    #         y_p[j] = float(L[1])
    #         z_p[j] = float(L[2])
    #         j = j+1
    
    # Calculate distorsion
    'We compute the difference in coordinates of each atom'
    'between the CONTCAR and the reference CONTCAR.'
    loop = int(nM * (n_layer/n_layer_tot))
    
    'loop is basically the number of metal atoms of interest,'
    'such that if n_layer = 1, we just take the surface atoms.'
    dist = np.zeros([ loop ])
    
    for i in range( loop ):
        dist[i] = module(x_ref[i+nM-loop], x[i+nM-loop],
                         y_ref[i+nM-loop], y[i+nM-loop],
                         z_ref[i+nM-loop], z[i+nM-loop],
                         x_size, y_size, z_size)
        
    'We average on the number of hydrogen atoms, or the number of surface metal atoms.'
    avg_M[ipt,1] = sum(dist)/nH
    print(avg_M[ipt,1])
    
    # 'For the hydrogen atoms, we compute the difference in'
    # 'coordinates between POSCAR and CONTCAR.'
    # dist_H = np.zeros([ nH ])
    
    # for i in range(nH):
    #     dist_H[i] = module(x_p[i], x[i+nM],
    #                        y_p[i], y[i+nM],
    #                        z_p[i], z[i+nM],
    #                        x_size, y_size, z_size)
    
    # 'We average on the number of hydrogen atoms.'
    # avg_H[ipt,1] = sum(dist_H)/nH
    
    # ipt = ipt+1

# %% Plot
coverage = avg_M[:,0] / nads

figure1 = plt.figure(1)

plt.plot(coverage, avg_M[:,1], marker='+', ms='10.0', linestyle=':',label=file)
# plt.plot(coverage, avg_H[:,1], marker='+', ms='10.0', linestyle=':',label='H')

plt.xlabel('Coverage 'r'$\theta$')
plt.ylabel('Average deformation length / Å')

plt.legend()
plt.show()
